;IR Detector modified for one detector and ICSP
;09.18.06 Modified to invert logic signal from original because
;common emitter level shifter interface introduces an inversion
	LIST P = 12F509, F = INHX8M
	INCLUDE "P12F509.INC"  
;==========================================================================
;       Verify Processor
;==========================================================================
        IFNDEF __12F509
           MESSG "Processor-header file mismatch.  Verify selected processor."
        ENDIF
;==========================================================================
    	;__CONFIG	( _MCLRE_OFF & _CP_OFF & _WDT_OFF & _IntRC_OSC )
	__config 0FFA
;==========================================================================
;       Program Variables
;==========================================================================

#define LED		 	GPIO,2		; Pin 5 = GP2 = Bit 2
#define LOGIC		GPIO,1		; Pin 6 = GP1 = Bit 1
#define INHIBIT		GPIO,3  	; Pin 4 = GP3 = Bit 3  
#define IRM_OP		GPIO,5		; Pin 2 = GP5 = Bit 5
#define IRED		GPIO,4		; Pin 3 = GP4 = Bit 4 
#define LLIMIT		D'30'		; Lower limit of allowable hits in a window
#define LHITLIM		D'5'		; Lower hits limit
#define UMISLIM		D'20'		; Upper miss limit

		LIST
WAIT		EQU	D'9' 	;  Location for counter for time delay 
FAKEHIT		EQU	D'14'	;  Counter for false detections
TRUEHIT		EQU	D'15' 	;  Counter for detections found properly
FAKEHIS		EQU	D'17'	;  History of bad hits to overcome noise
SERHIT		EQU	D'11' 	;  Number of hits in a row (SERIAL)
SERMIS		EQU	D'13'	;  Number of misses in a row

;================================
;       Main Code               
;================================
	ORG	0x1FF
	;MOVLW	0x50		; My EPROM calibration value,remove for OTPs
start
	ORG	0x000
	MOVWF 	OSCCAL		; Store the factory osc. calibration value
	MOVLW	B'11101000'	; Set pins 4,6 as inputs, 2,3,5,7 as outputs
	TRIS	GPIO		; Configure pins as either I or O
	MOVLW	B'00000000'	; Set OPTION bits (weak pullups not enabled)
	OPTION				; Implement OPTION bits
	CLRF	FAKEHIS		; History of bad hits
	CLRF	SERHIT		; Serial good hits history
	CLRF	SERMIS		; Serial misses history
	MOVLW	B'11001010'	; Make sure that GP1 and GP3 are '1' on read
	MOVWF	GPIO
main
	CALL	delay		; Look for false hits when IR LEDs are off
	CALL	pulse		; Pulse the left IR LED at 38 KHz
	MOVF	FAKEHIS,W	; Get current "noise" level
	SUBWF	TRUEHIT,W		; TRUEHIT - FAKEHIS => W
	BTFSS	STATUS,C	; C=1 means TRUEHIT >= FAKEHIS
	GOTO 	no_obj	 	; IR detector didn't get a hit through the noise	
	MOVLW	LLIMIT		; Get the lower limit for allowable hits
	SUBWF	TRUEHIT,W		; TRUEHIT - LLIMIT => W
	BTFSS	STATUS,C	; C=1 means that TRUEHIT >= LLIMIT = good hit
	GOTO	no_obj		; Not good enough
;================================
; SUBROUTINES                    
;================================
obj_det
	INCF	SERHIT,F	; Increment number of good hits
	DECF	SERMIS,F	; Decrement miss counter
	MOVLW	LHITLIM		; Lower hit limit
	SUBWF	SERHIT,W	; Enough good ones in a row?
	BTFSS	STATUS,C	; C=1 means SERHIT >= LHITLIM
	GOTO	main
	BSF		LED			; Turn on the left Visible LED
	BSF		LOGIC
	CLRF	SERMIS		; Clear miss counter
	GOTO	main
no_obj
	INCF	SERMIS,F	; Increment miss counter
	DECF	SERHIT,F	; Reduce hit counter	
	MOVLW	UMISLIM		; Upper missed ones limit
	SUBWF	SERMIS,W	; Too many bad ones in a row?
	BTFSS	STATUS,C	; C=1 means SERMIS >= UMISLIM
	GOTO	main
	BCF		LED			; Turn off the left visible LED
	BCF		LOGIC
	CLRF	SERHIT		; clear the hits counter
	GOTO	main
delay		
	MOVLW	D'100'	 
	MOVWF	WAIT		; Creates a delay
	CLRF	FAKEHIT		; Clear out for checking
d_loop
	BCF		IRED		; Make sure IR LEDs are off
	NOP					
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP		
    NOP		
    NOP		
    NOP		
    NOP		
	BTFSS 	IRM_OP		; Look for a false hit
	INCF	FAKEHIT,F	; Note any false hit
    NOP		
    NOP		
	DECFSZ 	WAIT,F	
	GOTO	d_loop	
	MOVF	FAKEHIS,W	;Get historic false readings
	SUBWF	FAKEHIT,W	;FAKEHIT - FAKEHIS => W
	BTFSS	STATUS,C	;if C=1 then FAKEHIT >= FAKEHIS
	GOTO	newless		;Go here if FAKEHIT < FAKEHIS
	MOVF	FAKEHIT,W	;Get current # of false hits
	MOVWF	FAKEHIS		;Save the new higher number
	RETLW	0			
newless
	RRF		FAKEHIT,F	
	RRF		FAKEHIT,W	
	ANDLW	0x3f		;Divide by 4
	SUBWF	FAKEHIS,F	;FAKEHIS - (FAKEHIT/4) => FAKEHIS, save lower number
	RETLW	0
pulse		
	MOVLW	D'100'		
	MOVWF	WAIT		;  Pulses IRED at 38 KHz
	CLRF	TRUEHIT		;  Clear count for now
p_loop
	BSF		IRED		  
	NOP			
	NOP			
	NOP			
	NOP			
	NOP			
	NOP			
	NOP			
	NOP			
	NOP			
	NOP			 
	NOP			
	BCF		IRED	
	NOP			
	NOP			
	NOP			
	NOP			
	NOP			
	NOP			
	BTFSS	IRM_OP		; Look for a detection
	INCF	TRUEHIT,F	; Record true hits    
	NOP			
	NOP			
	DECFSZ	WAIT,F		
	GOTO 	p_loop	 
	RETLW	0		
	
	END
	


